#!/usr/bin/env python

import wx
import os

# AQoPA gui imports
from aqopa.gui.modules_panel_gui import ModulesPanel, EVT_MODULES_CHANGED
from aqopa.gui.mmv_panel_gui import MMVPanel
from aqopa.gui.run_panel_gui import RunPanel, EVT_MODEL_PARSED
from aqopa.gui.results_panel_gui import ResultsPanel

"""
@file       main_notebook_gui.py
@brief      GUI for the main notebook, where we attach our AQoPA tabs
@author     Damian Rusinek
@date       created on 05-09-2013 by Damian Rusinek
@date       edited on 07-05-2014 by Katarzyna Mazur (visual improvements mainly)
"""

# modules communication events
ModuleSimulationRequestEvent, EVT_MODULE_SIMULATION_REQUEST = wx.lib.newevent.NewEvent() # Parameters: module
ModuleSimulationAllowedEvent, EVT_MODULE_SIMULATION_ALLOWED = wx.lib.newevent.NewEvent() # Parameters: interpreter
ModuleSimulationFinishedEvent, EVT_MODULE_SIMULATION_FINISHED = wx.lib.newevent.NewEvent()

class MainNotebook(wx.Notebook):
    """ """

    def __init__(self, parent):
        wx.Notebook.__init__(self, parent)

        ###########
        # MODULES
        ###########

        self.availableModules = []

        from aqopa.module import timeanalysis
        timeanalysis_module = timeanalysis.Module()
        timeanalysis_module.get_gui().Bind(EVT_MODULE_SIMULATION_REQUEST,
                                           self.OnModuleSimulationRequest)
        timeanalysis_module.get_gui().Bind(EVT_MODULE_SIMULATION_FINISHED,
                                           self.OnModuleSimulationFinished)
        self.availableModules.append(timeanalysis_module)

        from aqopa.module import energyanalysis
        m = energyanalysis.Module(timeanalysis_module)
        self.availableModules.append(m)

        from aqopa.module import reputation
        self.availableModules.append(reputation.Module())

        # list containing notebook images:
        # .ico seem to be more OS portable, although we use .png here
        # the (20, 20) is the size in pixels of the images
        il = wx.ImageList(20, 20)
        modelsTabImg = il.Add(wx.Bitmap(self.CreatePath4Resource('models_lib.png'), wx.BITMAP_TYPE_PNG))
        metricsTabImg = il.Add(wx.Bitmap(self.CreatePath4Resource('metrics.png'), wx.BITMAP_TYPE_PNG))
        versionsTabImg = il.Add(wx.Bitmap(self.CreatePath4Resource('versions.png'), wx.BITMAP_TYPE_PNG))
        runTabImg = il.Add(wx.Bitmap(self.CreatePath4Resource('run.png'), wx.BITMAP_TYPE_PNG))
        modulesTabImg = il.Add(wx.Bitmap(self.CreatePath4Resource('modules.png'), wx.BITMAP_TYPE_PNG))
        resultsTabImg = il.Add(wx.Bitmap(self.CreatePath4Resource('results.png'), wx.BITMAP_TYPE_PNG))
        self.AssignImageList(il)

        ###########
        # TABS
        ###########

        self.modelTab = MMVPanel(self)
        self.modelTab.Layout()
        self.Bind(wx.EVT_TEXT, self.OnModelTextChange, self.modelTab.dataTextArea)
        self.AddPage(self.modelTab, "Model")
        self.SetPageImage(0, modelsTabImg)

        self.metricsTab = MMVPanel(self)
        self.metricsTab.Layout()
        self.Bind(wx.EVT_TEXT, self.OnModelTextChange, self.metricsTab.dataTextArea)
        self.AddPage(self.metricsTab, "Metrics")
        self.SetPageImage(1, metricsTabImg)

        self.versionsTab = MMVPanel(self)
        self.versionsTab.Layout()
        self.Bind(wx.EVT_TEXT, self.OnModelTextChange, self.versionsTab.dataTextArea)
        self.versionsTab.Layout()
        self.AddPage(self.versionsTab, "Versions")
        self.SetPageImage(2, versionsTabImg)

        self.modulesTab = ModulesPanel(self, modules=self.availableModules)
        self.modulesTab.Bind(EVT_MODULES_CHANGED, self.OnModulesChange)
        self.modulesTab.Layout()
        self.AddPage(self.modulesTab, "Modules")
        self.SetPageImage(3, modulesTabImg)

        self.runTab = RunPanel(self)
        self.runTab.SetAllModules(self.availableModules)
        self.runTab.Layout()
        self.runTab.Bind(EVT_MODEL_PARSED, self.OnModelParsed)
        self.AddPage(self.runTab, "Run")
        self.SetPageImage(4, runTabImg)

        self.resultsTab = ResultsPanel(self)
        self.resultsTab.Layout()
        self.AddPage(self.resultsTab, "Results")
        self.SetPageImage(5, resultsTabImg)

    def CreatePath4Resource(self, resourceName):
        """
        @brief      creates and returns path to the
                    given file in the resource
                    ('assets') dir
        @return     path to the resource
        """
        tmp = os.path.split(os.path.dirname(__file__))
        return os.path.join(tmp[0], 'bin', 'assets', resourceName)

    def LoadModelFile(self, filePath):
        self.modelTab.dataTextArea.LoadFile(filePath)

    def LoadMetricsFile(self, filePath):
        self.metricsTab.dataTextArea.LoadFile(filePath)

    def LoadVersionsFile(self, filePath):
        self.versionsTab.dataTextArea.LoadFile(filePath)

    def SetModelData(self, data):
        self.modelTab.dataTextArea.SetValue(data)

    def SetMetricsData(self, data):
        self.metricsTab.dataTextArea.SetValue(data)

    def SetVersionsData(self, data):
        self.versionsTab.dataTextArea.SetValue(data)

    def GetModelData(self):
        return self.modelTab.dataTextArea.GetValue().strip()

    def GetMetricsData(self):
        return self.metricsTab.dataTextArea.GetValue().strip()

    def GetVersionsData(self):
        return self.versionsTab.dataTextArea.GetValue().strip()

    def OnModelTextChange(self, event):
        self.runTab.SetModel(self.GetModelData(),
                             self.GetMetricsData(),
                             self.GetVersionsData())
        event.Skip()

    def OnModulesChange(self, event):
        self.runTab.SetSelectedModules(event.modules)
        self.resultsTab.SetSelectedModules(event.modules)

    def OnModelParsed(self, event):
        self.resultsTab.ClearResults()
        event.Skip()

    def OnModuleSimulationRequest(self, event):
        """ """
        gui = event.module.get_gui()
        self.runTab.runButton.Enable(False)
        self.runTab.parseButton.Enable(False)

        wx.PostEvent(gui, ModuleSimulationAllowedEvent(interpreter=self.runTab.interpreter))

    def OnModuleSimulationFinished(self, event):
        """ """
        self.runTab.parseButton.Enable(True)


#!/usr/bin/env python

import sys
from aqopa.simulator.state import Hook

"""
@file       console.py
@author     Katarzyna Mazur
"""

class PrintResultsHook(Hook):

    def __init__(self, module, simulator, output_file=sys.stdout):
        """ """
        self.module = module
        self.simulator = simulator
        self.output_file = output_file

    def execute(self, context, **kwargs):
        """ """

        self.output_file.write('-'*80)
        self.output_file.write('\n')
        self.output_file.write('Module\tCarbon Dioxide Emissions Analysis (pounds of CO2 produced per kWh)')
        self.output_file.write('\n')
        self.output_file.write('Version\t%s\n\n' % self.simulator.context.version.name)

        # temp default value
        pounds_of_co2_per_kWh = 1.85

        emissions = self.module.calculate_all_emissions(self.simulator, context.hosts, pounds_of_co2_per_kWh)

        minemission, minhost = self.module.get_min_emission(self.simulator, context.hosts)
        maxemission, maxhost = self.module.get_max_emission(self.simulator, context.hosts)
        totalemission = self.module.get_total_emission(self.simulator, context.hosts)
        avgemission = self.module.get_avg_emission(self.simulator, context.hosts)

        self.output_file.write('Minimal emission:\t{0}\tHost: {1:}\t\n'.format(str(minemission), minhost.name))
        self.output_file.write('Maximal emission: \t{0}\tHost: {1:}\t\n'.format(str(maxemission), maxhost.name))
        self.output_file.write('Total emission:\t\t{0}\n'.format(str(totalemission)))
        self.output_file.write('Average emission:\t{0}\n'.format(str(avgemission)))

        self.output_file.write("\nActual emissions:\n")
        for host in context.hosts:
            self.output_file.write('{0}\t\t{1:}\t'.format(host.name, str(emissions[host])))
            self.output_file.write("\n")


#Module abstract

class Module():
    """
    Abstract class of module
    """
    
    def get_gui(self):
        """
        Method returns class that is used by GUI version of AQoPA.
        """
        return None
    
    def extend_model_parser(self, parser):
        """
        Method is called before parsing the qopml model file.
        Module can extend parser and add tokens, rules, etc.
        Method returns parser.
        """
        return parser
    
    def extend_metrics_parser(self, parser):
        """
        Method is called before parsing the qopml metrics file.
        Module can extend parser and add tokens, rules, etc.
        Method returns parser.
        """
        return parser
    
    def extend_config_parser(self, parser):
        """
        Method is called before parsing the qopml config file.
        Module can extend parser and add tokens, rules, etc.
        Method returns parser.
        """
        return parser
    
    def install_console(self, simulator):
        """
        Method is called before running simulation in console mode.
        Module installs itself in simulator. 
        It can register hooks, executors, etc.
        """
        return simulator
    
    def install_gui(self, simulator):
        """
        Method is called before running simulation in GUI mode.
        Module installs itself in simulator. 
        It can register hooks, executors, etc.
        """
        return simulator


'''
Created on 22-04-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''

from aqopa.model.parser.lex_yacc import LexYaccParserExtension
from aqopa.model import AlgWhile, AlgCallFunction, AlgIf, AlgReturn, AlgAssignment


class Builder():
    """
    Builder for store objects
    """
    


class ModelParserExtension(LexYaccParserExtension):
    """
    Extension for parsing functions
    """
    
    def __init__(self):
        LexYaccParserExtension.__init__(self)
        self.builder = Builder()
        self.open_blocks_cnt = 0

    ##########################################
    #           RESERVED WORDS
    ##########################################
    
    def word_algorithms_specification(self, t):
        t.lexer.push_state('algorithms')
        return t
    
    ##########################################
    #                TOKENS
    ##########################################

    def token_block_open(self, t):
        r'{'
        self.open_blocks_cnt += 1
        return t

    def token_block_close(self, t):
        r'}'
        self.open_blocks_cnt -= 1
        if self.open_blocks_cnt == 0:
            t.lexer.pop_state()
        return t
    
    ##########################################
    #                RULES
    ##########################################
    
    def algorithms_specification(self, t):
        """
        specification : ALGORITHMS_SPECIFICATION BLOCKOPEN algorithms_list BLOCKCLOSE
        """
        pass

    def algorithms_list(self, t):
        """
        algorithms_list : algorithm
                        | algorithms_list algorithm
        """
        pass

    def algorithm(self, t):
        """
        algorithm : ALGORITHM IDENTIFIER LPARAN IDENTIFIER RPARAN BLOCKOPEN algorithm_instructions BLOCKCLOSE
        """
        self.parser.store.algorithms[t[2]] = {'parameter': t[4], 'instructions': t[7]}

    def algorithm_instructions(self, t):
        """
        algorithm_instructions : algorithm_instruction
                                | algorithm_instructions algorithm_instruction
        """
        if len(t) == 3:
            t[0] = t[1]
            t[0].append(t[2])
        else:
            t[0] = []
            t[0].append(t[1])

    def algorithm_instruction(self, t):
        """
        algorithm_instruction : algorithm_instruction_assignment SEMICOLON
                            | algorithm_instruction_return SEMICOLON
                            | algorithm_instruction_if
                            | algorithm_instruction_while
        """
        t[0] = t[1]

    def algorithm_instruction_assignment(self, t):
        """
        algorithm_instruction_assignment : IDENTIFIER EQUAL algorithm_expression
        """
        t[0] = AlgAssignment(identifier=t[1], expression=t[3])

    def algorithm_instruction_return(self, t):
        """
        algorithm_instruction_return : RETURN algorithm_expression
        """
        t[0] = AlgReturn(expression=t[2])

    def algorithm_instruction_if(self, t):
        """
        algorithm_instruction_if : IF LPARAN algorithm_expression_conditional RPARAN BLOCKOPEN algorithm_instructions BLOCKCLOSE
            | IF LPARAN algorithm_expression_conditional RPARAN BLOCKOPEN algorithm_instructions BLOCKCLOSE ELSE BLOCKOPEN algorithm_instructions BLOCKCLOSE
        """
        if len(t) == 8:
            t[0] = AlgIf(condition=t[3], true_instructions=t[6], false_instructions=[])
        else:
            t[0] = AlgIf(condition=t[3], true_instructions=t[6], false_instructions=t[10])

    def algorithm_instruction_while(self, t):
        """
        algorithm_instruction_while : WHILE LPARAN algorithm_expression_conditional RPARAN BLOCKOPEN algorithm_instructions BLOCKCLOSE
        """
        t[0] = AlgWhile(condition=t[3], instructions=t[6])

    def algorithm_expression_conditional_comparison(self, t):
        """
        algorithm_expression_conditional : algorithm_expression EQUAL EQUAL algorithm_expression
                                        | algorithm_expression EXCLAMATION EQUAL algorithm_expression
                                        | algorithm_expression GREATER algorithm_expression
                                        | algorithm_expression GREATER EQUAL algorithm_expression
                                        | algorithm_expression SMALLER algorithm_expression
                                        | algorithm_expression SMALLER EQUAL algorithm_expression
                                        | algorithm_expression_conditional AND AND algorithm_expression_conditional
                                        | algorithm_expression_conditional OR OR algorithm_expression_conditional
        """
        t[0] = t[1]
        sign = t[2]
        if len(t) == 5:
            sign += t[3]
            t[0].append(sign)
            t[0].extend(t[4])
        else:
            t[0].append(sign)
            t[0].extend(t[3])

    def algorithm_expression_conditional_paran(self, t):
        """
        algorithm_expression_conditional : LPARAN algorithm_expression_conditional RPARAN
        """
        t[0] = t[2]
        t[0].prepend('(')
        t[0].append(')')

    def algorithm_expression_simple(self, t):
        """
        algorithm_expression : number
                            | IDENTIFIER
        """
        t[0] = [t[1]]

    def algorithm_expression_uminus(self, t):
        """
        algorithm_expression : MINUS algorithm_expression %prec UMINUS
        """
        t[0] = t[2]
        t[0].prepend('--')

    def algorithm_expression_paran(self, t):
        """
        algorithm_expression : LPARAN algorithm_expression RPARAN
        """
        t[0] = t[2]
        t[0].prepend('(')
        t[0].append(')')

    def algorithm_expression_operations(self, t):
        """
        algorithm_expression : algorithm_expression PLUS algorithm_expression
                            | algorithm_expression MINUS algorithm_expression
                            | algorithm_expression TIMES algorithm_expression
                            | algorithm_expression DIVIDE algorithm_expression
        """
        t[0] = t[1]
        t[0].append(t[2])
        t[0].extend(t[3])

    def algorithm_expression_function(self, t):
        """
        algorithm_expression : QUALITY LPARAN RPARAN
                                | SIZE LPARAN IDENTIFIER RPARAN
                                | SIZE LPARAN IDENTIFIER SQLPARAN INTEGER SQRPARAN RPARAN
        """
        args = []
        if len(t) == 5:
            args = [t[3]]
        elif len(t) == 8:
            args = [t[3], t[5]]
        t[0] = [AlgCallFunction(t[1], args)]

    #   Predefined functions
    
    def _extend(self):
        
        self.parser.add_state('algorithms', 'inclusive')

        self.parser.add_reserved_word('algorithms', 'ALGORITHMS_SPECIFICATION', func=self.word_algorithms_specification)
        self.parser.add_reserved_word('alg', 'ALGORITHM', state='algorithms')
        self.parser.add_reserved_word('if', 'IF', state='algorithms', case_sensitive=True)
        self.parser.add_reserved_word('while', 'WHILE', state='algorithms', case_sensitive=True)
        self.parser.add_reserved_word('else', 'ELSE', state='algorithms', case_sensitive=True)
        self.parser.add_reserved_word('quality', 'QUALITY', state='algorithms', case_sensitive=True)
        self.parser.add_reserved_word('size', 'SIZE', state='algorithms', case_sensitive=True)
        self.parser.add_reserved_word('return', 'RETURN', state='algorithms', case_sensitive=True)

        self.parser.add_token('BLOCKOPEN', func=self.token_block_open, states=['algorithms'])
        self.parser.add_token('BLOCKCLOSE', func=self.token_block_close, states=['algorithms'])
        self.parser.add_token('ARROWRIGHT', r'\-\>', states=['algorithms'])
        self.parser.add_token('ARROWLEFT', r'\<\-', states=['algorithms'])
        self.parser.add_token('ARROWBOTH', r'\<\-\>', states=['algorithms'])

        self.parser.add_token('PLUS', r'\+', states=['algorithms'])
        self.parser.add_token('MINUS', r'\-', states=['algorithms'])
        self.parser.add_token('TIMES', r'\*', states=['algorithms'])
        self.parser.add_token('DIVIDE', r'/', states=['algorithms'])

        self.parser.add_token('GREATER', r'\>', states=['algorithms'])
        self.parser.add_token('SMALLER', r'\<', states=['algorithms'])
        self.parser.add_token('EXCLAMATION', r'\!', states=['algorithms'])
        self.parser.add_token('AND', r'\&', states=['algorithms'])
        self.parser.add_token('OR', r'\|', states=['algorithms'])

        self.parser.add_rule(self.algorithms_specification)
        self.parser.add_rule(self.algorithms_list)
        self.parser.add_rule(self.algorithm)
        self.parser.add_rule(self.algorithm_instructions)
        self.parser.add_rule(self.algorithm_instruction)
        self.parser.add_rule(self.algorithm_instruction_assignment)
        self.parser.add_rule(self.algorithm_instruction_return)
        self.parser.add_rule(self.algorithm_instruction_if)
        self.parser.add_rule(self.algorithm_instruction_while)
        self.parser.add_rule(self.algorithm_expression_conditional_comparison)
        self.parser.add_rule(self.algorithm_expression_conditional_paran)
        self.parser.add_rule(self.algorithm_expression_simple)
        self.parser.add_rule(self.algorithm_expression_uminus)
        self.parser.add_rule(self.algorithm_expression_paran)
        self.parser.add_rule(self.algorithm_expression_operations)
        self.parser.add_rule(self.algorithm_expression_function)

        self.parser.add_precedence(['PLUS', 'MINUS'], 'left')
        self.parser.add_precedence(['TIMES', 'DIVIDE'], 'left')
        self.parser.add_precedence(['UMINUS'], 'right')


'''
Created on 01-06-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''
from aqopa.simulator.error import RuntimeException

class TimeSynchronizationException(RuntimeException):
    """
    Error occurs when the current times of hosts are not synchronized
    and they want to exchange messages.
    For example host A tries to send message to host B in time t1, but
    host B is already in time t2 > t1.
    """
    pass
    


'''
Created on 07-05-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''
from aqopa.model import BooleanExpression, IdentifierExpression,\
    CallFunctionExpression, TupleExpression, TupleElementExpression,\
    AssignmentInstruction, CommunicationInstruction, FinishInstruction,\
    ContinueInstruction, CallFunctionInstruction, IfInstruction,\
    WhileInstruction, HostSubprocess, COMMUNICATION_TYPE_OUT,\
    original_name, name_indexes, BreakInstruction

from aqopa.simulator.error import RuntimeException

class Context():
    
    def __init__(self, version):
        
        self.version = version
        
        self.hosts = []                 # List of all hosts in this context
        self.functions = []             # List of all functions in this context
        
        self.expression_populator = None
        self.expression_checker = None
        self.expression_reducer = None

        self.metrics_manager = None
        self.channels_manager = None
        self.algorithms_resolver = None
        
        self._current_host_index = 0
        self._previous_host_index = -1
    
    def get_current_host(self):
        """
        Return host being executed at this step.
        """
        return self.hosts[self._current_host_index]
    
    def get_current_instruction(self):
        """
        Return instruction being executed at this step.
        """
        return self.get_current_host().get_current_instructions_context().get_current_instruction()
    
    def all_hosts_finished(self):
        """
        Returns True if all hosts are in FINISHED state
        and no next states can be generated.
        """
        for h in self.hosts:
            if not h.finished():
                return False
        return True
    
    def get_progress(self):
        """
        Returs a number between 0 and 1 representing the progress of
        simulation.
        """
        all = 0
        ended = 0
        for h in self.hosts:
            if h.finished():
                ended += 1
            all += 1
        return float(ended)/float(all) if all > 0 else 0
    
    def has_epoch_ended(self):
        """
        Returns True when all hosts finished their epoch. Each host tried to execute an instruction
        in all its instruction contexts.
        Going to the next state is done for one host in next state generation step, so to finish
        the one loop for all hosts' instruction contexts (let N), simulator must call function N times.
        """
        for h in self.hosts:
            if not h.finished():
                if not h.has_epoch_ended():
                    return False
        return True

    def any_host_changed(self):
        """
        Return True in any host has changed in last next state generation loop performed 
        for all hosts.
        """
        for h in self.hosts:
            if h.has_changed():
                return True
        return False
    
    def mark_all_hosts_unchanged(self):
        """
        Sets the state of all hosts to unchanged.
        Used before each next state generation loop.
        """
        for h in self.hosts:
            h.mark_unchanged()
    
    def goto_next_host(self):
        """
        Context is moved to the next host so that next state generation step is performed for next host.
        """
        self._previous_host_index = self._current_host_index
        self._current_host_index = (self._current_host_index + 1) % len(self.hosts)

        # Go to next host until current host is not finihed
        while self._current_host_index != self._previous_host_index and self.get_current_host().finished():
            self._current_host_index = (self._current_host_index + 1) % len(self.hosts)


# ----------- Hook

HOOK_TYPE_PRE_HOST_LIST_EXECUTION       = 1
HOOK_TYPE_PRE_INSTRUCTION_EXECUTION     = 2
HOOK_TYPE_POST_INSTRUCTION_EXECUTION    = 3
HOOK_TYPE_SIMULATION_FINISHED           = 4

class Hook():
    """
    Hooks are executed in many places of simulation.
    Hooks can be added by modules.
    """
    
    def execute(self, context, **kwargs):
        """
        Method changes the context. 
        
        May return exectution result if used in executors hook.
        """
        raise NotImplementedError()
        
# ----------- Host Process

HOST_STATUS_RUNNING = 1
HOST_STATUS_FINISHED = 2

class Host():
    """
    Simulation equivalent of host
    """
    
    def __init__(self, name, instructions_list, predefined_variables=None):
        self.name = name
        self.instructions_list = instructions_list

        self._variables = {}
        if predefined_variables is not None:
            self._variables = predefined_variables
        
        self._scheduler = None
        self._changed = False
        self._status = HOST_STATUS_RUNNING 
        self._finish_error = None

        self._touches = 0 # How many times the host has been touched.
        # When touches is greater than number of instruction contexts, the epoch end.
        
    def __unicode__(self):
        return u"host %s" % unicode(self.name)

    def original_name(self):
        """"""
        return original_name(self.name)
        
    def add_name_index(self, index):
        """
        Add index to the name. 
        Before: name = ch, index = 1. After: name = ch.1
        """
        self.name += ".%d" % index
        
    def get_name_index(self):
        """
        Returns repetition index of this host
        """
        elems = self.name.split('.')
        if len(elems) < 2:
            return 0
        return int(elems[1])
        
    def set_scheduler(self, scheduler):
        """Set scheduler"""
        self._scheduler = scheduler
        
    def set_variable(self, name, value):
        """Set hotst's variable"""
        self._variables[name] = value
        
    def get_variable(self, name):
        """ Get host's variable """
        if name not in self._variables:
            raise RuntimeException("Variable '%s' undefined in host '%s'." % (name, self.name))
        return self._variables[name]

    def has_variable(self, name):
        """ Returns True if variable is defined """
        return name in self._variables

    def get_variables(self):
        """ Get variables dict """
        return self._variables

    def touch(self):
        """
        Host is touched before the execution of its instruction.
        It is used to check if all processes were tried to execute - hence whether the epoc has ended.
        Each time is touching is started when the first instruction context is executed.
        """
        # Touches equal to zero means that new epoch has been started.
        # The counter is started when the forst context is executed.
        if self._touches == 0:
            if self._scheduler.is_current_context_first():
                self._touches += 1
        else:
            self._touches += 1

    def has_epoch_ended(self):
        """
        Returns True when host tried to execute each instructions context in current epoch.
        """
        return self._touches > self._scheduler.get_unfinished_contexts_number()

    def mark_changed(self):
        """ Marks host changed. Means that host have changes in last state. """
        self._changed = True
        
    def mark_unchanged(self):
        """ Marks host unchanged. Clears changes from last state. """
        self._changed = False
        self._touches = 0
        
    def has_changed(self):
        """ Returns True if host has changed """
        return self._changed
        
    def goto_next_instructions_context(self):
        """
        Moves host to next instructions context.
        """
        self._scheduler.goto_next_instruction_context()
        if self._scheduler.finished() and not self.finished():
            self.finish_successfuly()
        
    def get_current_instructions_context(self):
        """
        Returns the current instructions context retrieved from scheduler.
        """
        return self._scheduler.get_current_context()

    def get_all_instructions_contexts(self):
        """
        Returns all instructions context retrived from scheduler.
        """
        return self._scheduler.get_all_contexts()

    def get_instructions_context_of_instruction(self, instruction):
        """
        Returns instruction context with the instruction from parameter
        as the current instruction
        """
        return self._scheduler.get_instructions_context_of_instruction(instruction)
    
    def get_current_process(self):
        """ """
        return self.get_current_instructions_context().get_process_of_current_list()

    def check_if_finished(self):
        """
        Method checks if host has no more instructions to execute
        and updates host's status.
        """
        if self._scheduler.finished():
            if self._status == HOST_STATUS_RUNNING:
                self._status = HOST_STATUS_FINISHED
                self._finish_error = None

    def finished(self):
        """ 
        Returns True when host is finished
        """
        return self._status == HOST_STATUS_FINISHED
    
    def finish_successfuly(self):
        """
        Finish host execution without error
        """
        self._status = HOST_STATUS_FINISHED
        self._finish_error = None
    
    def finish_failed(self, error):
        """
        Finish host execution with error
        """
        self._status = HOST_STATUS_FINISHED
        self._finish_error = error
        
    def get_finish_error(self):
        """ """
        return self._finish_error
    
    
class Process():
    
    def __init__(self, name, instructions_list):
        self.name = name
        self.instructions_list = instructions_list
    
        self.follower = None
        
    def original_name(self):
        """"""
        return original_name(self.name)
        
    def add_name_index(self, index):
        """
        Add index to the name. 
        Before: name = ch, index = 1. After: name = ch.1
        """
        self.name += ".%d" % index
        
    def __unicode__(self):
        return u"process %s" % unicode(self.name)
        
# ----------- Instructions Context

class InstructionsList:
    
    def __init__(self, instructions_list, process=None):
        self.process = process 
        self.instructions_list = instructions_list
        self._current_instruction_index = 0
        
    def get_current_instruction(self):
        """ """
        return self.instructions_list[self._current_instruction_index]
    
    def goto_next_instruction(self):
        """ """
        self._current_instruction_index += 1
        
    def finished(self):
        """
        Returns True if list is finished. 
        """
        return self._current_instruction_index >= len(self.instructions_list)

class InstructionsContext:
    
    def __init__(self, host):
        self.host = host
        self.stack = []        # Stack of instructions list
        
    def _get_current_list(self):
        """
        Returns currently executed list of instructions.
        """
        if len(self.stack) == 0:
            return None
        return self.stack[len(self.stack)-1]
        
    def get_current_instruction(self):
        """
        Returns currently executed instruction.
        """
        return self._get_current_list().get_current_instruction() if self._get_current_list() is not None else None
        
    def get_process_of_current_list(self):
        """
        Returns the process that current list is in.
        """
        return self._get_current_list().process if self._get_current_list() is not None else None
        
    def add_instructions_list(self, instructions_list, process=None):
        """
        Add instructions list to the stack.
        """
        l = InstructionsList(instructions_list, process)
        self.stack.append(l)
        
    def goto_next_instruction(self):
        """
        Moves context to the next instruction.
        """
        # Go to next instruction in current list
        self._get_current_list().goto_next_instruction()

        # Check if process has follower and if it should be started now
        current_process = self.get_process_of_current_list()
        if current_process is not None and current_process.follower is not None:
            # We will move to next instruction only if the stack has been popped at least once and no follower was added
            move_to_next_instruction = False
            while len(self.stack) > 0 and self._get_current_list().finished():
                self.stack.pop()
                # Stack is popped so we set move flag to True, because in upper instructions list
                # the pointer should be moved
                move_to_next_instruction = True
                new_current_process = self.get_process_of_current_list()
                # Compare processes between and after stack pop
                # If they are different it means that process has been finished
                if new_current_process != current_process:
                    # Add follower's instructions to the context
                    instructions_list = current_process.follower.instructions_list
                    # If follower has at least one instruction
                    if len(instructions_list) > 0:
                        self.add_instructions_list(instructions_list, current_process.follower)
                        # When new instructions are added to the context it should not be moved to the next instruction
                        # because we would omit the first new instruction
                        move_to_next_instruction = False
                        break
            if not self.finished() and move_to_next_instruction:
                # If context is still not finished and new current list is not LOOP
                # Go to next instruction in new current list
                if not isinstance(self._get_current_list().get_current_instruction(), WhileInstruction):
                    self._get_current_list().goto_next_instruction()

        # Now we handle the situation when current process has no follower

        # While current list is finished but context is not finished
        while not self.finished() and self._get_current_list().finished():
            # Remove current list
            self.stack.pop()
            # If context is still not finished and new current list is not LOOP
            # Go to next instruction in new current list
            if not self.finished():
                if not isinstance(self._get_current_list().get_current_instruction(), WhileInstruction):
                    self._get_current_list().goto_next_instruction()
            # And repeat this process
        # When moving to the next instruction in instructions context is finished
        # update host's status - it may become finished
        self.host.check_if_finished()

    def finished(self):
        """
        Returns True if context is finished.
        """
        if len(self.stack) == 0:
            return True
        
        if len(self.stack) == 1 and self._get_current_list().finished():
            return True
        
        return False
        
        
# ----------- Executor
    
class InstructionExecutor():
    """
    Abstract class of executor from the chain of responsibility.
    """
    
    def can_execute_instruction(self, instruction):
        """
        Returns True if instruction can be executed by executor.
        """
        raise NotImplementedError()
    
    def execute_instruction(self, context, **kwargs):
        """
        Changes the context according to the execution of current instruction.
        The second parameter `kwargs` keeps variables created be previous executors.
        Returns the result of execution containing information 
        about execution and the next steps: should it omit next executors?,
        does it consume cpu?, does it implement custom index management? etc.
        """
        raise NotImplementedError()
    
class ExecutionResult():
    """
    The result of execution of one instruction by one executor.
    """
    
    def __init__(self, consumes_cpu=False,
                 custom_index_management=False,
                 finish_instruction_execution=False,
                 result_kwargs=None):
        """ """
        self.consumes_cpu = consumes_cpu
        self.custom_index_management = custom_index_management
        self.finish_instruction_execution = finish_instruction_execution
        self.result_kwargs = result_kwargs
    
class HookExecutor(InstructionExecutor):
    """
    Class executes hooks.
    """
    
    def __init__(self):
        self._hooks = []     # List of hooks
    
    def add_hook(self, hook):
        """ Adds hook to the list """
        self._hooks.append(hook)
        return self
    
    def remove_hook(self, hook):
        """ Removes hook from the list """
        self._hooks.remove(hook)
        return self
    
    
    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        consumes_cpu = False 
        custom_index_management = False 
        finish_instruction_execution = False

        exec_kwargs = kwargs
        for h in self._hooks:
            result = h.execute(context, **exec_kwargs)
            if result:
                if result.consumes_cpu:
                    consumes_cpu = True
                if result.custom_index_management:
                    custom_index_management = True
                if result.finish_instruction_execution:
                    finish_instruction_execution = True
                if result.result_kwargs is not None:
                    exec_kwargs.update(result.result_kwargs)

                    
        return ExecutionResult(consumes_cpu, custom_index_management, 
                               finish_instruction_execution,
                               result_kwargs=exec_kwargs)
        
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return True
        
class PrintExecutor(InstructionExecutor):
    """
    Excecutor writes current instruction to the stream.
    """
    
    def __init__(self, f):
        self.file = f       # File to write instruction to
        
        self.result = ExecutionResult()
        
    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        self.file.write("Host: %s \t" % context.get_current_host().name)
        instruction = context.get_current_instruction()
        simples = [AssignmentInstruction, CommunicationInstruction, FinishInstruction, ContinueInstruction]
        for s in simples:
            if isinstance(instruction, s):
                self.file.write(unicode(instruction))
                
        if isinstance(instruction, CallFunctionInstruction):
            self.file.write(instruction.function_name + '(...)')
            
        if isinstance(instruction, IfInstruction):
            self.file.write('if (%s) ...' % unicode(instruction.condition))
            
        if isinstance(instruction, WhileInstruction):
            self.file.write('while (%s) ...' % unicode(instruction.condition))
            
        if isinstance(instruction, Process):
            self.file.write('process %s ...' % unicode(instruction.name))
            
        if isinstance(instruction, HostSubprocess):
            self.file.write('subprocess %s ...' % unicode(instruction.name))
        self.file.write("\n") 
        
        return self.result
        
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return True
        
class AssignmentInstructionExecutor(InstructionExecutor):
    """
    Executes assignment insructions.
    """
    
    def _compute_current_expression(self, expression, context):
        """
        Computes the expression from current assignment instruction.
        """
        if isinstance(expression, BooleanExpression):
            return expression.clone()
        
        if isinstance(expression, IdentifierExpression) or isinstance(expression, CallFunctionExpression) \
                or isinstance(expression, TupleExpression) or isinstance(expression, TupleElementExpression):
            # Population of variables which were previously populated (ie. additional attrs)
            return context.expression_populator.populate(expression, context.get_current_host())

        raise RuntimeException("Expression '%s' cannot be a value of variable.")

    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        instruction = context.get_current_instruction()
        expression = self._compute_current_expression(instruction.expression, context)
        expression = context.expression_reducer.reduce(expression)

        context.get_current_host().set_variable(instruction.variable_name, expression)
        context.get_current_host().mark_changed()
        
        return ExecutionResult(consumes_cpu=True)
        
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, AssignmentInstruction)
    
class CallFunctionInstructionExecutor(InstructionExecutor):
    """
    Executes call function insructions.
    Dummy executor just to show that call function instruction 
    consumes cpu and changes host.
    """
    
    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        context.get_current_host().mark_changed()
        
        return ExecutionResult(consumes_cpu=True)
        
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, CallFunctionInstruction)
    
class ProcessInstructionExecutor(InstructionExecutor):
    """
    Executes process insructions.
    """

    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        process_instruction = context.get_current_instruction()
        instructions_list = process_instruction.instructions_list

        # If process has at least one instruction
        if len(instructions_list) > 0:
            context.get_current_host().get_current_instructions_context().add_instructions_list(instructions_list,
                                                                                                process_instruction)
        else:
            # Go to next instruction if proces has no instructions
            context.get_current_host().get_current_instructions_context().goto_next_instruction()

        context.get_current_host().mark_changed()

        return ExecutionResult(custom_index_management=True)
        
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, Process)
    
class SubprocessInstructionExecutor(InstructionExecutor):
    """
    Executes subprocess insructions.
    """

    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        subprocess_instruction = context.get_current_instruction()
        current_process = context.get_current_host().get_current_process()
        instructions_list = subprocess_instruction.instructions_list

        if len(instructions_list) > 0:
            context.get_current_host().get_current_instructions_context().add_instructions_list(instructions_list, current_process)
        else:
            context.get_current_host().get_current_instructions_context().goto_next_instruction()
            
        context.get_current_host().mark_changed()

        return ExecutionResult(custom_index_management=True)
                
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, HostSubprocess)
    

class CommunicationInstructionExecutor(InstructionExecutor):
    """
    Executes communication in-out insructions.
    """
    
    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        instruction = context.get_current_instruction()
        channel = context.channels_manager.find_channel_for_current_instruction(context)

        if context.channels_manager.find_channel(instruction.channel_name) is None:
            raise RuntimeException("Channel {0} undefined.".format(instruction.channel_name))

        if not channel:
            context.get_current_host().get_current_instructions_context().goto_next_instruction()
            context.get_current_host().mark_changed()
            return ExecutionResult(consumes_cpu=True,
                                   custom_index_management=True)
        
        if instruction.communication_type == COMMUNICATION_TYPE_OUT:
            if kwargs is not None and 'sent_message' in kwargs:
                message = kwargs['sent_message']
            else:
                message = context.channels_manager.build_message(
                    context.get_current_host(),
                    context.get_current_host().get_variable(instruction.variable_name).clone(),
                    context.expression_checker)
                kwargs['sent_message'] = message
            channel.send_message(context.get_current_host(), message, context.channels_manager.get_router())

            # Go to next instruction
            context.get_current_host().get_current_instructions_context().goto_next_instruction()
            context.get_current_host().mark_changed()
            
        else:
            if kwargs is not None and 'messages_request' in kwargs:
                request = kwargs['messages_request']
            else:
                request = context.channels_manager.build_message_request(context.get_current_host(), instruction,
                                                                         context.expression_populator)
                kwargs['messages_request'] = request

            channel.wait_for_message(request)
            
        return ExecutionResult(consumes_cpu=True, 
                               custom_index_management=True,
                               result_kwargs=kwargs)
                
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, CommunicationInstruction)
    
class FinishInstructionExecutor(InstructionExecutor):
    """
    Executes finish (end, stop) insructions.
    """
    
    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        instruction = context.get_current_instruction()
        if instruction.command == "end":
            for h in context.hosts:
                h.finish_successfuly()
        else:
            msg = 'Executed stop instruction in host {0}'.format(context.get_current_host().name)
            for h in context.hosts:
                if h == context.get_current_host():
                    h.finish_failed('Executed stop instruction')
                else:
                    h.finish_failed(msg)
            
        context.get_current_host().mark_changed()
        
        return ExecutionResult(consumes_cpu=True)
        
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, FinishInstruction)
    
class ContinueInstructionExecutor(InstructionExecutor):
    """
    Executes continue insructions.
    """
    
    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        instructions_context = context.get_current_host().get_current_instructions_context()
        instruction = instructions_context.get_current_instruction()
        while instruction and not isinstance(instruction, WhileInstruction):
            instructions_context.stack.pop()
            instruction = instructions_context.get_current_instruction()

        context.get_current_host().mark_changed()

        return ExecutionResult(consumes_cpu=True, 
                               custom_index_management=True)
                
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, ContinueInstruction)

class BreakInstructionExecutor(InstructionExecutor):
    """
    Executes continue insructions.
    """

    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        instructions_context = context.get_current_host().get_current_instructions_context()
        instruction = instructions_context.get_current_instruction()
        while instruction and not isinstance(instruction, WhileInstruction):
            instructions_context.stack.pop()
            instruction = instructions_context.get_current_instruction()
        instructions_context.goto_next_instruction()

        context.get_current_host().mark_changed()

        return ExecutionResult(consumes_cpu=True,
                               custom_index_management=True)

    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, BreakInstruction)
    
class IfInstructionExecutor(InstructionExecutor):
    """
    Executes if-clause insructions.
    """
    
    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        instruction = context.get_current_instruction()
        current_process = context.get_current_host().get_current_process()
        
        contidion_result = context.expression_checker.result(instruction.condition, 
                                        context.get_current_host())

        if contidion_result:
            instructions_list = instruction.true_instructions
        else:
            instructions_list = instruction.false_instructions

        if len(instructions_list) > 0:
            context.get_current_host().get_current_instructions_context().add_instructions_list(
                                                                            instructions_list, 
                                                                            current_process)
        else:
            context.get_current_host().get_current_instructions_context().goto_next_instruction()
        
        context.get_current_host().mark_changed()

        return ExecutionResult(consumes_cpu=True, 
                               custom_index_management=True)
                
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, IfInstruction)
    
    
class WhileInstructionExecutor(InstructionExecutor):
    """
    Executes while-clause insructions.
    """
    
    def execute_instruction(self, context, **kwargs):
        """ Overriden """
        instruction = context.get_current_instruction()
        current_process = context.get_current_host().get_current_process()

        contidion_result = context.expression_checker.result(instruction.condition,
                                        context.get_current_host())
        
        if contidion_result:
            instructions_list = instruction.instructions
            
            if len(instructions_list) > 0:
                context.get_current_host().get_current_instructions_context().add_instructions_list(
                                                                                instructions_list, 
                                                                                current_process)
        else:
            context.get_current_host().get_current_instructions_context().goto_next_instruction()
        
        context.get_current_host().mark_changed()

        return ExecutionResult(consumes_cpu=True, 
                               custom_index_management=True)
                
    def can_execute_instruction(self, instruction):
        """ Overriden """
        return isinstance(instruction, WhileInstruction)
    
    
class Executor():
    """
    Class executes instructions to move simulation to the nest state.
    """
    
    def __init__(self):
        self.executors = []     # List of instruction executors
    
    def prepend_instruction_executor(self, instruction_executor):
        """
        Adds instruction executor at the beginning of the executors list.
        """
        self.executors.insert(0, instruction_executor)
    
    def append_instruction_executor(self, instruction_executor):
        """
        Adds instruction executor to the end of the executors list.
        """
        self.executors.append(instruction_executor)
    
    def execute_instruction(self, context):
        """
        Executes one instruction of context which is equal to going to the next state.
        The executed instruction is get from the current host of the context.
        """
        
        execution_result = None
        cpu_time_consumed = False
        # Execute instructions in one instructions context until 
        # instruction that consumes cpu time is executes.
        # Method also checks whether the host was not stopped or ended meanwhile
        # and whether the context is not finished 
        # (fe. when there is only information about instructions printed)
        while not cpu_time_consumed and not context.get_current_host().finished() \
                and not context.get_current_host().get_current_instructions_context().finished():
            
            instr = context.get_current_instruction()
            custom_instructions_index_change = False

            exec_kwargs = {}
            for e in self.executors:
                if e.can_execute_instruction(instr):
                    
                    # Execute current instruction by current executor. 
                    # Executor can change index.
                    execution_result = e.execute_instruction(context, **exec_kwargs)
                    
                    # If executor does not return result
                    # create a default one
                    if not execution_result:
                        execution_result = ExecutionResult()
                    
                    if execution_result.consumes_cpu:
                        cpu_time_consumed = True
                        
                    # Check if executor changes instructions index itself.
                    if execution_result.custom_index_management:
                        custom_instructions_index_change = True

                    if execution_result.result_kwargs is not None:
                        exec_kwargs.update(execution_result.result_kwargs)

                # Omit other executors if the result says that
                if execution_result.finish_instruction_execution:
                    break

            # If index is not changed by executor,
            # method has to change it. 
            if not custom_instructions_index_change:
                context.get_current_host().get_current_instructions_context().goto_next_instruction()
           
            # Finish execution of instruction
            if execution_result and execution_result.finish_instruction_execution:
                break
           
        # Change the index of instructions in current host
        # according to the scheduler algorithm.
        # It moves index to next instructions context.
        context.get_current_host().goto_next_instructions_context()
    


#!/usr/bin/env python

import wx
import wx.lib.newevent
from aqopa.gui.combo_check_box import ComboCheckBox
from aqopa.gui.general_purpose_frame_gui import GeneralFrame

"""
@file       modules_panel_gui.py
@brief      GUI for the 'Modules' tab on AQoPA's main window (panel)
@author     Damian Rusinek <damian.rusinek@gmail.com>
@date       created on 05-09-2013 by Damian Rusinek
@date       edited on 07-05-2014 by Katarzyna Mazur
"""

ModulesChangedEvent, EVT_MODULES_CHANGED = wx.lib.newevent.NewEvent()

class ModulesPanel(wx.Panel):
    """
    Panel used for selecting modules and configuring them.
    """

    def __init__(self, *args, **kwargs):
        self.allModules = kwargs['modules']
        del kwargs['modules']

        wx.Panel.__init__(self, *args, **kwargs)

        # our main sizer
        mainSizer = wx.BoxSizer(wx.VERTICAL)

        # 'Select' button = select chosen modules
        self.selectButton = wx.Button(self, label="Select")
        # 'Configure' button = configure selected module, clicking the button should bring a new window where the configuration of the chosen module will be possible
        self.configureButton = wx.Button(self, label="Configure")
        self.configureButton.Disable()

        # create group boxes, aka static boxes
        modulesSelectionBox = wx.StaticBox(self, label="Select modules")
        modulesConfigurationBox = wx.StaticBox(self, label="Configure modules")
        mainBox = wx.StaticBox(self, label="Modules")

        # create sizers = some kind of layout management
        modulesSelectionBoxSizer = wx.StaticBoxSizer(modulesSelectionBox, wx.HORIZONTAL)
        modulesConfigurationBoxSizer = wx.StaticBoxSizer(modulesConfigurationBox, wx.HORIZONTAL)
        mainBoxSizer = wx.StaticBoxSizer(mainBox, wx.VERTICAL)

        # create labels, aka static texts
        selectModulesLabel = wx.StaticText(self, label="Choose modules for analysis and click the 'Select'\nbutton to add them to the configuration panel.")
        configureModulesLabel = wx.StaticText(self, label="Choose module from selected modules and\nconfigure them one by one.")

        # create combocheckbox, empty at first
        self.comboCheckBox = wx.combo.ComboCtrl(self)
        self.tcp = ComboCheckBox()
        self.comboCheckBox.SetPopupControl(self.tcp)
        self.comboCheckBox.SetText('...')

        # create ordinary combobox for module configuration
        self.modulesConfComboBox = wx.ComboBox(self, style=wx.TE_READONLY)

        # add tooltipz = make user's life easier
        modulesSelectionBox.SetToolTip(wx.ToolTip("Select modules for analysis"))
        modulesConfigurationBox.SetToolTip(wx.ToolTip("Configure chosen modules"))

        # align 'select modules' group box
        modulesSelectionBoxSizer.Add(selectModulesLabel, 1, wx.ALL | wx.EXPAND, 5)
        modulesSelectionBoxSizer.Add(self.comboCheckBox, 1, wx.ALL | wx.EXPAND, 5)
        modulesSelectionBoxSizer.Add(self.selectButton, 0, wx.ALL | wx.EXPAND, 5)

        # align 'configure modules' group box
        modulesConfigurationBoxSizer.Add(configureModulesLabel, 1, wx.ALL | wx.EXPAND, 5)
        modulesConfigurationBoxSizer.Add(self.modulesConfComboBox, 1, wx.ALL | wx.EXPAND, 5)
        modulesConfigurationBoxSizer.Add(self.configureButton, 0, wx.ALL | wx.EXPAND, 5)

        # do some bindings:
        self.selectButton.Bind(wx.EVT_BUTTON, self.OnSelectButtonClicked)
        self.configureButton.Bind(wx.EVT_BUTTON, self.OnConfigureButtonClicked)

        #self.tcp.checkBoxList.Bind(wx.EVT_CHECKLISTBOX, self.OnCheckBoxChange)

        # names of the modules to appeared in the config combobox
        self.modulesNames4Combo = []

        for m in self.allModules:
            gui = m.get_gui()
            self.modulesNames4Combo.append(gui.get_name())

        # fill combocheckbox with modules names
        self.tcp.SetChoices(self.modulesNames4Combo)

        for i in range(0,4) :
            mainSizer.Add(wx.StaticText(self), 0, 0, wx.ALL | wx.EXPAND, 5)
        mainSizer.Add(modulesSelectionBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        for i in range(0,3) :
            mainSizer.Add(wx.StaticText(self), 0, 0, wx.ALL | wx.EXPAND, 5)
        mainSizer.Add(modulesConfigurationBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        mainBoxSizer.Add(mainSizer, 0, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(mainBoxSizer)

    def OnSelectButtonClicked(self, event):
        """
        @brief grabs selected modules from
        combocheckbox widget, puts them into
        conf combobox
        """

        # add selected modules to config-combo box - u can
        # configure only selected modules, one by one
        self.FillUpComboWithModules(self.tcp.GetSelectedItems())
        # check which module was selected, make a list out of the selected modules
        modules = []
        [modules.append(self.allModules[i]) for i in range(self.tcp.checkBoxList.GetCount()) if self.tcp.checkBoxList.IsChecked(i)]
        # perform event - modules were selected
        wx.PostEvent(self, ModulesChangedEvent(modules=modules, all_modules=self.allModules))

    def OnConfigureButtonClicked(self, event):
        """
        @brief configures chosen module [ideally, do it in
        a new window]
        """
        # print self.modulesConfComboBox.GetStringSelection()
        # get selected module from combo
        selectedModule = self.modulesConfComboBox.GetValue()

        for m in self.allModules :
            if m.get_gui().get_name() == selectedModule :
                # new window (frame, actually) where we open up a
                # panel received from module/name/gui.py[get_configuration_panel]
                confWindow = GeneralFrame(self, "Module Configuration", "Configuration", "config.png")
                panel = m.get_gui().get_configuration_panel(confWindow)
                confWindow.AddPanel(panel)
                confWindow.Show()
                break

    def FillUpComboWithModules(self, modules):
        """
        @brief adds selected modules to the combobox
        """
        # clear combo, do not remember prev choices
        self.modulesConfComboBox.Clear()
        # DO NOT select, selecting can mess up many things
        self.modulesConfComboBox.SetSelection(-1)
        self.modulesConfComboBox.SetValue("")
        # add all chosen modules to the combo
        self.modulesConfComboBox.AppendItems(modules)
        self.modulesConfComboBox.Refresh()
        # enable button if combo is not empty =
        # we can actually configure some modules
        if modules :
            self.configureButton.Enable()
        else:
            self.configureButton.Disable()


#!/usr/bin/env python

import wx
import sys
import time
import traceback

# AQoPA imports
from aqopa import app
from aqopa.model.parser import MetricsParserException,\
    ConfigurationParserException, ModelParserException
from aqopa.simulator.error import EnvironmentDefinitionException,\
    RuntimeException

"""
@file       run_panel_gui.py
@brief      GUI for the 'Run' tab on AQoPA's main window (panel)
@author     Damian Rusinek <damian.rusinek@gmail.com>
@date       created on 05-09-2013 by Damian Rusinek
@date       edited on 08-05-2014 by Katarzyna Mazur (visual improvements)
"""

# model parsing events
ModelParsedEvent, EVT_MODEL_PARSED = wx.lib.newevent.NewEvent()
ModelParseErrorEvent, EVT_MODEL_PARSE_ERROR = wx.lib.newevent.NewEvent()

class RunPanel(wx.Panel):
    """ """
    def __init__(self, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)

        ###############
        # SIMULATION
        ###############

        self.qopml_model              = ""
        self.qopml_metrics            = ""
        self.qopml_configuration      = ""

        self.allModules = []
        self.selectedModules    = []

        self.interpreter        = None
        self.finishedSimulators = []

        self.progressTimer      = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.OnProgressTimerTick, self.progressTimer)

        ###############
        # LAYOUT
        ###############

        panelSizer = wx.BoxSizer(wx.HORIZONTAL)

        # create the top panel - it can be the parsing panel
        # (when parsing the model) or the run panel = when
        # running the simulation process (analysis)
        topPanel = wx.Panel(self, style=wx.ALIGN_CENTER)
        topPanel.SetSizer(panelSizer)

        # build panels
        self.parsingPanel = self._BuildParsingPanel(topPanel)
        self.runPanel = self._BuildRunPanel(topPanel)

        # align panels
        panelSizer.Add(self.parsingPanel, 1, wx.ALL | wx.EXPAND, 5)
        panelSizer.Add(self.runPanel, 1, wx.ALL | wx.EXPAND, 5)

        # hide the run panel (for now on) it will be visible
        # after parsing the model and clicking the 'run' button
        self.runPanel.Hide()

        # align panels
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(topPanel, 1, wx.ALL | wx.EXPAND, 5)

        # create the bottom panel - the one with the buttons
        bottomPanel = wx.Panel(self, style=wx.ALIGN_CENTER)

        # create buttons
        self.parseButton = wx.Button(bottomPanel, label="Parse")
        self.parseButton.SetToolTip(wx.ToolTip("Parse chosen model"))
        self.runButton = wx.Button(bottomPanel, label="Run")
        self.runButton.SetToolTip(wx.ToolTip("Start the simulation process"))
        self.runButton.Enable(False)
        #self.cleanButton = wx.Button(bottomPanel, label="Clean")

        # properly align the bottom panel, buttons on the right
        panelSizer = wx.BoxSizer(wx.HORIZONTAL)
        panelSizer.Add(self.parseButton, 0, wx.LEFT | wx.ALL, 5)
        panelSizer.Add(self.runButton, 0, wx.LEFT | wx.ALL, 5)
        #panelSizer.Add(self.cleanButton, 0, wx.LEFT | wx.ALL, 5)
        bottomPanel.SetSizer(panelSizer)
        sizer.Add(bottomPanel, 0, wx.ALIGN_RIGHT | wx.RIGHT, 5)

        self.SetSizer(sizer)

        ###############
        # EVENTS
        ###############

        self.parseButton.Bind(wx.EVT_BUTTON, self.OnParseClicked)
        self.runButton.Bind(wx.EVT_BUTTON, self.OnRunClicked)
        #self.cleanButton.Bind(wx.EVT_BUTTON, self.OnCleanClicked)

        self.Bind(EVT_MODEL_PARSED, self.OnModelParsed)

    def _BuildParsingPanel(self, parent):
        """
        @brief      creates the parsing panel
        @return     brand new parsing panel
        """

        # new panel, we will return it and set as the main panel
        # for AQoPA's main windows's Run tab
        panel = wx.Panel(parent)

        # create group boxes aka static boxes
        parsingInfoBox = wx.StaticBox(panel, label="Model parsing information")
        # create sizers = some kind of layout management
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        parsingInfoBoxSizer = wx.StaticBoxSizer(parsingInfoBox, wx.HORIZONTAL)
        # create text area for displaying parsing information
        self.parseResult = wx.TextCtrl(panel, style=wx.TE_MULTILINE | wx.TE_READONLY)

        # do the final panel alignment
        parsingInfoBoxSizer.Add(self.parseResult, 1, wx.ALL | wx.EXPAND, 5)
        sizer.Add(parsingInfoBoxSizer, 1, wx.EXPAND, 5)
        panel.SetSizer(sizer)

        return panel

    def _BuildRunPanel(self, parent):
        """
        @brief      creates the run panel
        @return     brand new run panel
        """

        panel = wx.Panel(parent)
        sizer = wx.BoxSizer(wx.VERTICAL)

        # create group boxes aka static boxes
        self.statusStaticBox = wx.StaticBox(panel, label="Status: ")
        timeStaticBox = wx.StaticBox(panel, label="Analysis Time")
        runInfoBox = wx.StaticBox(panel, label="Model run information")

        # create sizers = some kind of layout management
        self.statusStaticBoxSizer = wx.StaticBoxSizer(self.statusStaticBox, wx.VERTICAL)
        timeStaticBoxSizer = wx.StaticBoxSizer(timeStaticBox, wx.VERTICAL)
        runInfoBoxSizer = wx.StaticBoxSizer(runInfoBox, wx.VERTICAL)
        progressSizer = wx.BoxSizer(wx.HORIZONTAL)

        # create labels
        self.percentLabel = wx.StaticText(panel, label="0%")
        self.dotsLabel = wx.StaticText(panel, label=".")
        self.analysisTime = wx.StaticText(panel, label='---')

        # create text area
        self.runResult = wx.TextCtrl(panel, style=wx.TE_MULTILINE | wx.TE_READONLY)

        # add content to sizers
        progressSizer.Add(self.dotsLabel, 0, wx.ALIGN_LEFT, 5)
        progressSizer.Add(self.percentLabel, 0, wx.ALIGN_LEFT, 5)
        self.statusStaticBoxSizer.Add(progressSizer, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        timeStaticBoxSizer.Add(self.analysisTime, 0, wx.ALL | wx.ALIGN_CENTER, 5)
        runInfoBoxSizer.Add(self.runResult, 1, wx.ALL | wx.EXPAND, 5)
        sizer.Add(self.statusStaticBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(timeStaticBoxSizer, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(runInfoBoxSizer, 1, wx.ALL | wx.EXPAND, 5)

        # do the final alignment
        panel.SetSizer(sizer)

        return panel

    def ShowPanel(self, panel):
        self.parsingPanel.Hide()
        self.runPanel.Hide()
        panel.Show()
        self.Layout()

    def SetModel(self, model, metrics, configuration):
        """ """
        self.qopml_model              = model
        self.qopml_metrics            = metrics
        self.qopml_configuration      = configuration

    def SetSelectedModules(self, modules):
        """ """
        self.selectedModules = modules

    def SetAllModules(self, modules):
        """ """
        self.allModules = modules

    def OnParseClicked(self, event):
        """ """
        self.interpreter = app.GuiInterpreter(
                                     model_as_text=self.qopml_model,
                                     metrics_as_text=self.qopml_metrics,
                                     config_as_text=self.qopml_configuration)

        for m in self.selectedModules:
            self.interpreter.register_qopml_module(m)

        try:
            resultMessage = ""
            error = False
            self.interpreter.parse(self.allModules)
            resultMessage = "SUCCESFULLY PARSED\n\n Now you can run simulation."
            wx.PostEvent(self, ModelParsedEvent())
        except EnvironmentDefinitionException, e:
            error = True
            resultMessage = "ENVIRONMENT ERROR\n"
            resultMessage += "%s\n" % unicode(e)
        except ModelParserException, e:
            error = True
            resultMessage = "MODEL SYNTAX ERROR\n"
            if len(e.syntax_errors):
                resultMessage += "\n".join(e.syntax_errors)
        except MetricsParserException, e:
            error = True
            resultMessage = "METRICS SYNTAX ERROR\n"
            if len(e.syntax_errors):
                resultMessage += "\n".join(e.syntax_errors)
        except ConfigurationParserException, e:
            error = True
            resultMessage = "VERSIONS SYNTAX ERROR\n"
            if len(e.syntax_errors):
                resultMessage += "\n".join(e.syntax_errors)

        if error:
            resultMessage += "\nModel may include syntax parsed by modules (eg. metrics, configuration). "+\
                             "Have you selected modules?"
            wx.PostEvent(self, ModelParseErrorEvent(error=resultMessage))
        if resultMessage != "":
            self.parseResult.SetValue(resultMessage)

        self.ShowPanel(self.parsingPanel)

    def OnModelParsed(self, event):
        """ """
        self.runButton.Enable(True)

    def OnRunClicked(self, event):
        """ """
        if not self.selectedModules:
            dial = wx.MessageDialog(None, "You have to choose some modules!", 'Warning', wx.OK | wx.ICON_EXCLAMATION)
            dial.ShowModal()
        else :
            try:
                self.DisableModulesSelection(True)

                self.statusStaticBox.SetLabel("Status: running")
                self.analysisTime.SetLabel("---")
                self.percentLabel.SetLabel("0%")
                self.runResult.SetValue("")

                self.runButton.Enable(False)
                self.ShowPanel(self.runPanel)

                self.finishedSimulators = []
                self.simulatorIndex = 0

                self.startAnalysisTime = time.time()

                self.interpreter.prepare()
                self.progressTimer.Start(1000)

                simulator = self.interpreter.simulators[self.simulatorIndex]
                wx.lib.delayedresult.startWorker(self.OnSimulationFinished,
                                                 self.interpreter.run_simulation,
                                                 wargs=(simulator,),
                                                 jobID = self.simulatorIndex)


            except EnvironmentDefinitionException, e:
                self.statusStaticBox.SetLabel("Status: error")
                self.runButton.Enable(True)
                errorMessage = "Error on creating environment: %s\n" % e
                if len(e.errors) > 0:
                    errorMessage += "Errors:\n"
                    errorMessage += "\n".join(e.errors)
                self.runResult.SetValue(errorMessage)
                self.progressTimer.Stop()

            except Exception, e:
                self.statusStaticBox.SetLabel("Error")
                self.runButton.Enable(True)
                sys.stderr.write(traceback.format_exc())
                errorMessage = "Unknown error\n"
                self.runResult.SetValue(errorMessage)
                self.progressTimer.Stop()

            self.ShowPanel(self.runPanel)

    def OnSimulationFinished(self, result):
        """ """
        simulator = self.interpreter.simulators[self.simulatorIndex]
        self.simulatorIndex += 1
        self.finishedSimulators.append(simulator)

        resultMessage = None
        try :
            simulator = result.get()

            self.PrintProgressbar(self.GetProgress())

            for m in self.selectedModules:
                gui = m.get_gui()
                gui.on_finished_simulation(simulator)

            runResultValue = self.runResult.GetValue()
            resultMessage = runResultValue + \
                "Version %s finished successfully.\n\n" \
                % simulator.context.version.name

        except RuntimeException, e:
            runResultValue = self.runResult.GetValue()
            resultMessage = runResultValue + \
                "Version %s finished with error: \nHost: %s \nInstruction: %s\nError: %s \n\n" \
                % (simulator.context.version.name,
                    unicode(simulator.context.get_current_host()),
                    unicode(simulator.context.get_current_instruction()),
                    e.args[0])
        except Exception, e:
            sys.stderr.write(traceback.format_exc())
            runResultValue = self.runResult.GetValue()
            resultMessage = runResultValue + \
                "Version %s finished with unknown error.\n\n" \
                % simulator.context.version.name

        if resultMessage:
            self.runResult.SetValue(resultMessage)

        if len(self.finishedSimulators) == len(self.interpreter.simulators):
            self.OnAllSimulationsFinished()
        else:
            simulator = self.interpreter.simulators[self.simulatorIndex]
            wx.lib.delayedresult.startWorker(self.OnSimulationFinished,
                                             self.interpreter.run_simulation,
                                             wargs=(simulator,),
                                             jobID = self.simulatorIndex)

    def OnAllSimulationsFinished(self):
        """ """
        self.DisableModulesSelection(False)
        self.progressTimer.Stop()
        self.PrintProgressbar(1)

        self.endAnalysisTime = time.time()
        timeDelta = self.endAnalysisTime - self.startAnalysisTime
        analysisTimeLabel = "%.4f s" % timeDelta

        self.analysisTime.SetLabel(analysisTimeLabel)
        self.Layout()

        for m in self.selectedModules:
            gui = m.get_gui()
            gui.on_finished_all_simulations(self.interpreter.simulators)

    ################
    # PROGRESS BAR
    ################

    def OnProgressTimerTick(self, event):
        """ """
        progress = self.GetProgress()
        if progress == 1:
            self.progressTimer.Stop()
        self.PrintProgressbar(progress)

    def GetProgress(self):
        all = 0.0
        sum = 0.0
        for simulator in self.interpreter.simulators:
            all += 1
            sum += simulator.context.get_progress()
        progress = 0
        if all > 0:
            progress = sum / all
        return progress

    def PrintProgressbar(self, progress):
        """
        Prints the formatted progressbar showing the progress of simulation.
        """
        percentage = str(int(round(progress*100))) + '%'
        self.percentLabel.SetLabel(percentage)
        self.runPanel.Layout()

        if progress == 1:
            self.statusStaticBox.SetLabel("Status: finished")
            self.runPanel.Layout()
            self.dotsLabel.SetLabel('')
            self.runPanel.Layout()
        else:
            dots = self.dotsLabel.GetLabel()
            if len(dots) > 10:
                dots = "."
            else:
                dots += ".."
            self.dotsLabel.SetLabel(dots)
            self.runPanel.Layout()

    #def OnCleanClicked(self, event):
    #    self.parseResult.Clear()

    def DisableModulesSelection(self, value) :

        """
        @brief  disables/enables elements on 'Modules' tab (panel),
        thanks to such approach we can get rid of some errors, simply
        disable modules selection/configuration when the simulation
        is running, and re-enable it when simulation ends
        """

        # get run panel parent - that is, the wx Notebook
        notebook = self.GetParent()
        # get modules panel, it's the third page in our wx Notebook
        modulesTab = notebook.GetPage(3)

        # disable or enable gui elements (depends on 'value')
        if value :
            modulesTab.selectButton.Disable()
            modulesTab.configureButton.Disable()
            modulesTab.comboCheckBox.Disable()
            modulesTab.modulesConfComboBox.Disable()
        else :
            modulesTab.selectButton.Enable()
            modulesTab.configureButton.Enable()
            modulesTab.comboCheckBox.Enable()
            modulesTab.modulesConfComboBox.Enable()


#!/usr/bin/env python

import wx
import wx.richtext

"""
@file       mmv_panel_gui.py
@brief      GUI for the 'Model', 'Metrics' and 'Versions' tabs on AQoPA's main window (panel)
@author     Damian Rusinek <damian.rusinek@gmail.com>
@date       created on 05-09-2013 by Damian Rusinek
@date       edited on 06-05-2014 by Katarzyna Mazur (visual improvements mainly)
"""

class MMVPanel(wx.Panel):
    """
    @brief      panel containing text area (and buttons 2) for one of model parts:
                model, metrics, configuration, used for creating tabs on AQoPA's
                main window
    """

    def __init__(self, *args, **kwargs):
        """
         @brief         Initializes and aligns all the gui elements for
                        tabs: model, metrics and versions
        """
        wx.Panel.__init__(self, *args, **kwargs)

        # create group boxes aka static boxes
        self.tabBox = wx.StaticBox(self, label="File: ")

        # create sizers = some kind of layout management
        self.tabBoxSizer = wx.StaticBoxSizer(self.tabBox, wx.HORIZONTAL)

        # create text area; here we display model, metric or version content
        self.dataTextArea = wx.richtext.RichTextCtrl(self, style=wx.TE_MULTILINE | wx.TE_NO_VSCROLL)

        # create buttons - simple 'Load' and 'Save' will be enough
        self.loadButton = wx.Button(self, label="Load")
        self.saveButton = wx.Button(self, label="Save")
        # or add the 3rd button maybe? clears the
        # content of model/metric/version if clicked
        self.cleanButton = wx.Button(self, label="Clear")

        # create label 4 displaying information about the cursor
        # position in the opened file
        self.cursorInfoLabel = wx.StaticText(self)

        # create checkbox - make model/metric/version editable / not editable
        self.editable = wx.CheckBox(self, -1, label="Editable")

        # do some bindings -
        # show cursor position below the text area
        self.dataTextArea.Bind(wx.EVT_KEY_UP, self.printCursorInfo)
        self.dataTextArea.Bind(wx.EVT_LEFT_UP, self.printCursorInfo)
        # bind buttons to appropriate actions
        self.loadButton.Bind(wx.EVT_BUTTON, self.OnLoadClicked)
        self.saveButton.Bind(wx.EVT_BUTTON, self.OnSaveClicked)
        self.cleanButton.Bind(wx.EVT_BUTTON, self.OnCleanClicked)
        # bind checkbox state with the appropriate action - simply
        # allow / do not allow to edit opened model/metric/version
        self.editable.Bind(wx.EVT_CHECKBOX, self.OnCheckBoxClicked)

        # at first, we are not allowed to edit
        self.editable.SetValue(False)
        # pretend that we clicked the check box, so it's event gets called
        wx.PostEvent(self.editable, wx.CommandEvent(wx.wxEVT_COMMAND_CHECKBOX_CLICKED))

        # align buttons and the checkbox
        bottomSizer = wx.BoxSizer(wx.HORIZONTAL)
        bottomSizer.Add(self.editable, 0, wx.EXPAND | wx.ALL, 5)
        # create empty static box (label) in order to make a horizontal
        # gap between the checkbox and the buttons
        bottomSizer.Add(wx.StaticText(self), 1, wx.EXPAND, 5)
        bottomSizer.Add(self.loadButton, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        bottomSizer.Add(self.saveButton, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        bottomSizer.Add(self.cleanButton, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        # add some 'useful' tooltips
        self.loadButton.SetToolTip(wx.ToolTip("Load from HDD"))
        self.saveButton.SetToolTip(wx.ToolTip("Save to HDD"))
        self.cleanButton.SetToolTip(wx.ToolTip("Clear all"))

        # add text area to the fancy group box with a filename above the displayed file content
        self.tabBoxSizer.Add(self.dataTextArea, 1, wx.EXPAND | wx.ALL, 5)

        # do the final alignment
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.tabBoxSizer, 1, wx.EXPAND | wx.ALL, 5)
        sizer.Add(self.cursorInfoLabel, 0, wx.EXPAND | wx.ALL, 5)
        sizer.Add(bottomSizer, 0, wx.EXPAND, 5)
        self.SetSizer(sizer)
        self.Layout()

    def OnLoadClicked(self, event):
        """ Load file to text area """
        ofdlg = wx.FileDialog(self, "Load file", "", "", "QoP-ML Files (*.qopml)|*.qopml",
                              wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
        ofdlg.ShowModal()
        if ofdlg.GetPath():
            wildcard, types = wx.richtext.RichTextBuffer.GetExtWildcard(save=False)
            fileType = types[ofdlg.GetFilterIndex()]
            self.dataTextArea.LoadFile(ofdlg.GetPath(), fileType)
        self.SetFilenameOnGUI(ofdlg.GetPath())
        ofdlg.Destroy()

    def OnSaveClicked(self, event):
        """ Save text area value to file """
        ofdlg = wx.FileDialog(self, "Save file", "", "", "QoP-ML Files (*.qopml)|*.qopml",
                              wx.FD_SAVE)
        ofdlg.ShowModal()
        if ofdlg.GetPath() :
            # add *.qopml extension if not given
            validated = self.ValidateFilename(ofdlg.GetPath())
            f = open(validated, "w")
            f.write(self.dataTextArea.GetValue())
            f.close()
            # save on gui name of the saved file (with *.qopml) extension
            self.SetFilenameOnGUI(validated)
        ofdlg.Destroy()

    def OnCleanClicked(self, event):
        self.dataTextArea.SetValue("")

    def printCursorInfo(self, event):
        """ """
        pos = self.dataTextArea.GetInsertionPoint()
        xy = self.dataTextArea.PositionToXY(pos)
        self.cursorInfoLabel.SetLabel("Line: %d, %d"
                                      % (xy[1]+1, xy[0]+1))
        self.Layout()
        event.Skip()

    def OnPaintPrettyPanel(self, event):
        # establish the painting canvas
        dc = wx.PaintDC(self)
        x = 0
        y = 0
        w, h = self.GetSize()
        dc.GradientFillLinear((x, y, w, h), '#606060', '#E0E0E0', nDirection=wx.NORTH)

    def SetFilenameOnGUI(self, filename):
        """
        @brief      sets the title of the group box
                    to the opened/saved filename
        """
        self.tabBox.SetLabel("File: "+filename)

    def ValidateFilename(self, filename):
        """
        @brief      adds *.qopml extension to the file which
                    we want 2 save (but only if not given)
        @return     returns the validated filename
        """
        if filename.endswith(".qopml") :
            return filename
        else :
            filename += ".qopml"
            return filename

    def OnCheckBoxClicked(self, event):
        """
        @brief      checks if the checkbox is checked :D
                    if so, you can edit opened model/metric/version
                    if not, model/metric/version is not editable,
                    nah, just a fancy feature
        """
        if event.IsChecked() :
            self.dataTextArea.SetEditable(True)
        else :
            self.dataTextArea.SetEditable(False)


'''
Created on 01-06-2013

@author: Damian Rusinek <damian.rusinek@gmail.com>
'''

from aqopa.model import MetricsServiceParam
from aqopa.model.parser.lex_yacc import LexYaccParserExtension


class Builder():
    
    def create_metrics_services_param_energy(self, token):
        """
        metrics_services_param : SQLPARAN ENERGY COLON metrics_services_energy_type LPARAN metrics_services_energy_unit RPARAN SQRPARAN
                               | SQLPARAN ENERGY COLON LISTENING COLON EXACT LPARAN metrics_services_energy_unit RPARAN SQRPARAN
        """
        if len(token) == 9:
            return MetricsServiceParam(token[2], token[4], token[6])
        else:
            param_name = token[4] + ':' + token[6]
            return MetricsServiceParam(token[2], param_name, token[6])

class ModelParserExtension(LexYaccParserExtension):
    """
    Extension for timeanalysis module communications time
    """

    #######################
    #  Communication Time
    #######################

    def sending_current_default_parameter(self, t):
        """
        medium_default_parameter : SENDING_CURRENT_DEFAULT_PARAMETER EQUAL comm_current_value
        """
        t[0] = {'default_sending_current': t[3]}

    def receiving_current_default_parameter(self, t):
        """
        medium_default_parameter : RECEIVING_CURRENT_DEFAULT_PARAMETER EQUAL comm_current_value
        """
        t[0] = {'default_receiving_current': t[3]}

    def listening_current_default_parameter(self, t):
        """
        medium_default_parameter : LISTENING_CURRENT_DEFAULT_PARAMETER EQUAL comm_current_value
        """
        t[0] = {'default_listening_current': t[3]}

    def topology_rule_sending_current_parameter(self, t):
        """
        topology_rule_parameter : SENDING_CURRENT_PARAMETER EQUAL comm_current_value
        """
        t[0] = {'sending_current': t[3]}

    def topology_rule_receiving_current_parameter(self, t):
        """
        topology_rule_parameter : RECEIVING_CURRENT_PARAMETER EQUAL comm_current_value
        """
        t[0] = {'receiving_current': t[3]}

    def topology_rule_listening_current_parameter(self, t):
        """
        topology_rule_parameter : LISTENING_CURRENT_PARAMETER EQUAL comm_current_value
        """
        t[0] = {'listening_current': t[3]}

    def comm_current_value(self, t):
        """
        comm_current_value : comm_current_metric
                            | comm_current_algorithm
        """
        t[0] = t[1]

    def comm_current_metric(self, t):
        """
        comm_current_metric : number comm_current_metric_unit
        """
        t[0] = {
            'type': 'metric',
            'value': t[1],
            'unit': t[2]
        }

    def comm_current_algorithm(self, t):
        """
        comm_current_algorithm : IDENTIFIER SQLPARAN comm_current_metric_unit SQRPARAN
                            | IDENTIFIER
        """
        unit = 'mA'
        if len(t) == 5:
            unit = t[3]
        t[0] = {
            'type': 'algorithm',
            'name': t[1],
            'unit': unit
        }

    def comm_current_metric_unit(self, t):
        """
        comm_current_metric_unit : MA
        """
        t[0] = t[1]

    def _extend(self):
        """ """

        self.parser.add_reserved_word('mA', 'MA', state='communication', case_sensitive=True)
        self.parser.add_reserved_word('default_sending_current', 'SENDING_CURRENT_DEFAULT_PARAMETER',
                                      state='communication',)
        self.parser.add_reserved_word('default_receiving_current', 'RECEIVING_CURRENT_DEFAULT_PARAMETER',
                                      state='communication',)
        self.parser.add_reserved_word('default_listening_current', 'LISTENING_CURRENT_DEFAULT_PARAMETER',
                                      state='communication',)
        self.parser.add_reserved_word('sending_current', 'SENDING_CURRENT_PARAMETER',
                                      state='communication',)
        self.parser.add_reserved_word('receiving_current', 'RECEIVING_CURRENT_PARAMETER',
                                      state='communication',)
        self.parser.add_reserved_word('listening_current', 'LISTENING_CURRENT_PARAMETER',
                                      state='communication',)

        self.parser.add_rule(self.sending_current_default_parameter)
        self.parser.add_rule(self.receiving_current_default_parameter)
        self.parser.add_rule(self.listening_current_default_parameter)
        self.parser.add_rule(self.topology_rule_sending_current_parameter)
        self.parser.add_rule(self.topology_rule_receiving_current_parameter)
        self.parser.add_rule(self.topology_rule_listening_current_parameter)
        self.parser.add_rule(self.comm_current_value)
        self.parser.add_rule(self.comm_current_metric)
        self.parser.add_rule(self.comm_current_algorithm)
        self.parser.add_rule(self.comm_current_metric_unit)

class ConfigParserExtension(LexYaccParserExtension):
    """
    Extension for timeanalysis module communications time
    """

    #######################
    #  Communication Time
    #######################

    def version_sending_current_default_parameter(self, t):
        """
        version_medium_default_parameter : SENDING_CURRENT_DEFAULT_PARAMETER EQUAL version_comm_current_value
        """
        t[0] = {'default_sending_current': t[3]}

    def version_receiving_current_default_parameter(self, t):
        """
        version_medium_default_parameter : RECEIVING_CURRENT_DEFAULT_PARAMETER EQUAL version_comm_current_value
        """
        t[0] = {'default_receiving_current': t[3]}

    def version_topology_rule_sending_current_parameter(self, t):
        """
        version_topology_rule_parameter : SENDING_CURRENT_PARAMETER EQUAL version_comm_current_value
        """
        t[0] = {'sending_current': t[3]}

    def version_topology_rule_receiving_current_parameter(self, t):
        """
        version_topology_rule_parameter : RECEIVING_CURRENT_PARAMETER EQUAL version_comm_current_value
        """
        t[0] = {'receiving_current': t[3]}

    def version_listening_current_default_parameter(self, t):
        """
        version_medium_default_parameter : LISTENING_CURRENT_DEFAULT_PARAMETER EQUAL version_comm_current_value
        """
        t[0] = {'default_listening_current': t[3]}

    def version_topology_rule_listening_current_parameter(self, t):
        """
        version_topology_rule_parameter : LISTENING_CURRENT_PARAMETER EQUAL version_comm_current_value
        """
        t[0] = {'listening_current': t[3]}

    def version_comm_current_value(self, t):
        """
        version_comm_current_value : version_comm_current_metric
                            | version_comm_current_algorithm
        """
        t[0] = t[1]

    def version_comm_current_metric(self, t):
        """
        version_comm_current_metric : number version_comm_current_metric_unit
        """
        t[0] = {
            'type': 'metric',
            'value': t[1],
            'unit': t[2]
        }

    def version_comm_current_algorithm(self, t):
        """
        version_comm_current_algorithm : IDENTIFIER SQLPARAN version_comm_current_metric_unit SQRPARAN
                                | IDENTIFIER
        """
        unit = 'mA'
        if len(t) == 5:
            unit = t[3]
        t[0] = {
            'type': 'algorithm',
            'name': t[1],
            'unit': unit
        }

    def version_comm_current_metric_unit(self, t):
        """
        version_comm_current_metric_unit : MA
        """
        t[0] = t[1]

    def _extend(self):
        """ """

        self.parser.add_reserved_word('mA', 'MA', state='versioncommunication', case_sensitive=True)
        self.parser.add_reserved_word('default_sending_current', 'SENDING_CURRENT_DEFAULT_PARAMETER',
                                      state='versioncommunication',)
        self.parser.add_reserved_word('default_receiving_current', 'RECEIVING_CURRENT_DEFAULT_PARAMETER',
                                      state='versioncommunication',)
        self.parser.add_reserved_word('default_listening_current', 'LISTENING_CURRENT_DEFAULT_PARAMETER',
                                      state='versioncommunication',)
        self.parser.add_reserved_word('sending_current', 'SENDING_CURRENT_PARAMETER',
                                      state='versioncommunication',)
        self.parser.add_reserved_word('receiving_current', 'RECEIVING_CURRENT_PARAMETER',
                                      state='versioncommunication',)
        self.parser.add_reserved_word('listening_current', 'LISTENING_CURRENT_PARAMETER',
                                      state='versioncommunication',)

        self.parser.add_rule(self.version_sending_current_default_parameter)
        self.parser.add_rule(self.version_receiving_current_default_parameter)
        self.parser.add_rule(self.version_topology_rule_sending_current_parameter)
        self.parser.add_rule(self.version_topology_rule_receiving_current_parameter)
        self.parser.add_rule(self.version_listening_current_default_parameter)
        self.parser.add_rule(self.version_topology_rule_listening_current_parameter)
        self.parser.add_rule(self.version_comm_current_value)
        self.parser.add_rule(self.version_comm_current_metric)
        self.parser.add_rule(self.version_comm_current_algorithm)
        self.parser.add_rule(self.version_comm_current_metric_unit)

class MetricsParserExtension(LexYaccParserExtension):
    """
    Extension for parsing energy analysis module's metrics
    """

    def __init__(self):
        LexYaccParserExtension.__init__(self)

        self.builder = Builder()


    ##################################
    #    Metrics energy consumption
    ##################################

    def metrics_services_param_energy(self, t):
        """
        metrics_services_param : SQLPARAN CURRENT COLON metrics_services_energy_type LPARAN metrics_services_energy_unit RPARAN SQRPARAN
        """
        t[0] = self.builder.create_metrics_services_param_energy(t)

    def metrics_services_energy_param_type(self, t):
        """
        metrics_services_energy_type : EXACT
        """
        t[0] = t[1].lower()

    def metrics_services_energy_unit(self, t):
        """
        metrics_services_energy_unit : MA
        """
        t[0] = t[1].lower()

    def _extend(self):
        """ """

        self.parser.add_reserved_word('current', 'CURRENT', state='metricsprimhead', case_sensitive=False)
        self.parser.add_reserved_word('exact', 'EXACT', state='metricsprimhead', case_sensitive=False)
        self.parser.add_reserved_word('mA', 'MA', state='metricsprimhead', case_sensitive=True)

        self.parser.add_rule(self.metrics_services_param_energy)
        self.parser.add_rule(self.metrics_services_energy_param_type)
        self.parser.add_rule(self.metrics_services_energy_unit)
